
#include "Direction.h"


Direction::Direction()
{

}


Direction::~Direction()
{
	
}

bool Direction::update(double step, const double curTime)
{
	bool DirectionEmpty = false;
	int NoEmptyLanes = 0;
	for(int i = 0; i < m_NoLanes; i++)
	{
		bool laneEmpty = m_vLanes[i].update(step);
		if(laneEmpty)
			NoEmptyLanes++;
	}
	
	// output any vehicles past the detector
	m_OutputDetector->WriteVehiclesToFile(curTime);

	if(NoEmptyLanes == m_NoLanes)
		DirectionEmpty = true;

	return DirectionEmpty;
}

void Direction::init(bool DirPos, Detector* OutDet, bool AllowLaneChange)
{
	m_DirPos = DirPos;
	m_OutputDetector = OutDet;
	m_AllowLaneChanging = AllowLaneChange;
}

void Direction::createLanes(int RoadLength, int nLanes, std::vector<Detector*> detectors)
{
	m_NoLanes = nLanes;

	for(int i = 0; i < m_NoLanes; i++)
	{
		Lane lane(RoadLength, detectors, m_DirPos, m_AllowLaneChanging);
		lane.setOutputDetector(m_OutputDetector);
		m_vLanes.push_back(lane);
	}

	// set adjacent lanes if more than 1 lane
	if(m_NoLanes > 1)
	{
		// If(DRIVE_ON_RIGHT), lanes are numbered from the BTM of the screen as:
		// Dir Pos - 0, 1, 2, 3 -  Dir Neg - 4, 5, 6, 7
		// else lanes are numbered from the TOP of the screen
		if(DRIVE_ON_RIGHT)
		{
			for(i = 0; i < m_NoLanes - 1; i++)
				m_vLanes.at(i).setLeftLane(&m_vLanes.at(i+1));	// next lane up the screen
			for(i = 1; i < m_NoLanes; i++)
				m_vLanes.at(i).setRightLane(&m_vLanes.at(i-1));	// one down the screen
		}
		else
		{
			for(i = 0; i < m_NoLanes - 1; i++)
				m_vLanes.at(i).setRightLane(&m_vLanes.at(i+1));	// next lane down the screen
			for(i = 1; i < m_NoLanes; i++)
				m_vLanes.at(i).setLeftLane(&m_vLanes.at(i-1));	// next one up the screen
		}
	}
}

M2D Direction::getGlobalPos()
{
	// this modifies the lane position to take accound of direction
	M2D vVeh2D = getVehiclesM2D();

	if(!m_DirPos)	// if in negative x-direction
	{
		for(int i = 0; i < m_NoLanes; i++)
		{
			int d2 = vVeh2D[i].size();
			for(int j = 0; j < d2; j++)
				vVeh2D[i][j]->setRoadPos( m_vLanes[i].getLength() - vVeh2D[i][j]->getPos() );
		}
	}
	return vVeh2D;
}

M2D Direction::getVehiclesM2D()
{
	// returns vehicles for position from the start of the lane extraction
	M2D vVeh2D;

	for(int i = 0; i < m_NoLanes; i++)
	{
		std::vector<Vehicle*> temp = m_vLanes[i].getPos();
		vVeh2D.push_back(temp);
	}
	return vVeh2D;
}

int Direction::getNoLanes()
{
	return m_NoLanes;
}

Lane& Direction::getLane(int i)
{
	return m_vLanes.at(i);
}

Detector* Direction::getOutputDetector()
{
	return m_OutputDetector;
}

void Direction::setOutputDetector(Detector *OutputDet)
{
	m_OutputDetector = OutputDet;
}
