
#include "DistValues.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//
//0 - Exponential
//1 - Log-normal
//2 - Gamma
//3 - Gumbel
//4 - Poisson
//5 - GEV
//6 - Normal
//

int DistValues::m_index= 0;

DistValues::DistValues():twopi(2.0 * 3.141592654)
{

	std::cout << "CDistValues::m_index - " << ++m_index << std::endl;
	PI = 4 * atan(1);
	
	unsigned long germe[6];
	for(int i = 0; i < 6; i++)
		germe[i] = static_cast<long> (time( NULL )); // 100;// for debugging
	RngStream::SetPackageSeed (germe);

	const int MAX_No_RNG = 4;	//note this has to be changed on the header file as well.
	RngStream temp[MAX_No_RNG];
	//g0, g1 and g2 used in box muller and calcValues g3 used as a general RNG elsewhere.

	for(i=0;i<MAX_No_RNG;i++)
	{
		g[i] = temp[i];
		g[i].IncreasedPrecis (true);	
		g[i].SetAntithetic (true);
	}
}

DistValues::~DistValues()
{


}

double DistValues::box_muller()
{
	/* normal random variate generator */
	
	double x1, x2, w, y1;
	static double y2;
	static int use_last = 0;

	if (use_last)		        /* use value from previous call */
	{
		y1 = y2;
		use_last = 0;
	}
	else
	{
		do {
			x1 = 2.0 * g[0].RandU01() - 1.0;
			x2 = 2.0 * g[1].RandU01() - 1.0;
			w = x1 * x1 + x2 * x2;
		} while ( w >= 1.0 );

		w = sqrt( (-2.0 * log( w ) ) / w );
		y1 = x1 * w;
		y2 = x2 * w;
		use_last = 1;
	}
	return( y1 );
}

double DistValues::RandU01()
{

	return g[3].RandU01();
}

int DistValues::RandInt(int i, int j)
{

	return g[3].RandInt(i,j);
}

double DistValues::calcValues(double Mean, double StDev, double Shape, int DistType)
{
	double x[2], u[3]; 
	double result = 0.0;
	double m = Mean; double sigma = StDev; double xi = Shape;

	for(int i = 0; i < 3; i++)
	{
		u[i] = g[i].RandU01();
	}

	switch(DistType)
	{
	case 0:
		result = -log(u[1]);
		break;
	case 1:
		x[0] = m + sigma * sqrt(-2 * log(u[0])) * cos(2 * PI * u[1]);
		x[1] = m + sigma * sqrt(-2 * log(u[0])) * sin(2 * PI * u[1]);
		result = exp( x[ (int)(u[1] + 0.5) ] );
		break;
	case 2:
		x[0] = (1 / m) * (-log(u[2])) * pow( u[0] , (1 / sigma) ) / ( pow( u[0] , (1 / sigma) ) + pow( u[1] , (1 / (1 - sigma)) ) );
		result = x[0];
		break;
	case 3:
		x[0] = m - (1 / sigma) * log(log(1 / u[0]));
		result = x[0];
		break;
	case 4:
		//if(m <= 10) cout << "Poisson Parameter < 10";
		m = m - 0.5;
		sigma = sqrt(sigma);
		x[0] = m + sigma * sqrt(-2 * log(u[0])) * cos(2 * PI * u[1]);
		x[1] = m + sigma * sqrt(-2 * log(u[0])) * sin(2 * PI * u[1]);
		result = x[ (int)(u[1] + 0.5) ];
		break;
	case 5:
		result = m + sigma * (1 - pow(-log(u[0]),xi)) / xi;
		break;
	default:		// normal distribution
		result = m + sigma * box_muller();
		break;
	}
	return result;
}