// EvolveTrafficView.cpp : implementation of the CEvolveTrafficView class
//

#include "stdafx.h"
#include "EvolveTraffic.h"

#include "EvolveTrafficDoc.h"
#include "EvolveTrafficView.h"
#include "SimConfigDlg.h"
#include "TrafficConfigDlg.h"
#include "PreferencesDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficView

IMPLEMENT_DYNCREATE(CEvolveTrafficView, CScrollView)

BEGIN_MESSAGE_MAP(CEvolveTrafficView, CScrollView)
	//{{AFX_MSG_MAP(CEvolveTrafficView)
	ON_COMMAND(ID_CONFIG_SIM, OnConfigSim)
	ON_COMMAND(ID_RUN_EVOLVETRAFFIC, OnRunEvolveTraffic)
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_COMMAND(ID_CONFIG_TRAF, OnConfigTraf)
	ON_COMMAND(ID_TOOLS_PREFS, OnToolsPrefs)
	ON_COMMAND(ID_TOOLS_PAUSE, OnToolsPause)
	ON_COMMAND(ID_TOOLS_ZOOMIN, OnToolsZoomin)
	ON_COMMAND(ID_TOOLS_ZOOMOUT, OnToolsZoomout)
	ON_COMMAND(ID_TOOLS_SPEEDUP, OnToolsSpeedup)
	ON_COMMAND(ID_TOOLS_SLOWDOWN, OnToolsSlowdown)
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficView construction/destruction

CEvolveTrafficView::CEvolveTrafficView()
{
	// The scale of the view is 1 meter per 1 pixel
	m_Scale = 1.0;
	
	m_Border_Top			= (20 /m_Scale);
	m_Border_Btm			= (40 /m_Scale);
	m_Border_Lhs			= (20 /m_Scale);
	m_Border_Rhs			= (100/m_Scale);
	m_TickLength			= (10 /m_Scale);
	m_TickStep				= (500/m_Scale);
	m_LaneWidth				= (20 /m_Scale);
	m_VehicleLengthScale	= (2  /m_Scale);	// so they look ok - results in overlaps??
	m_VehicleWidth			= (14 /m_Scale);	// less than lane width! 
	
	m_bTrucksReadIn = false;
	m_bInSimulation = false;	// has a simulation begun
	m_bTimerHasTicked = false;	// has the timer just ticked
	m_TimerID = 1;				// So we can kill/set from everywhere
	m_bAnimate = false;			// is the simulation to progress one step
	m_TimeWarp = 4;				// multiplier of real time that the simulation runs
	// the timer speed is constant since the refresh rate is constant
	m_TimerSpeed = Round(1000/DRAW_FRAMES_PER_SEC);
}

CEvolveTrafficView::~CEvolveTrafficView()
{

}

BOOL CEvolveTrafficView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficView drawing

void CEvolveTrafficView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = 100;
	SetScrollSizes(MM_TEXT, sizeTotal);

	m_pDoc = GetDocument();
	ASSERT_VALID(m_pDoc);
	
	initRoad();
}

void CEvolveTrafficView::initRoad()
{
	m_RoadLength	= m_pDoc->getRoadLength();
	m_NoLanes		= m_pDoc->getNoLanes();
	m_NoLanesDirPos	= m_pDoc->getNoLanesDirPos();
	m_NoLanesDirNeg	= m_pDoc->getNoLanesDirNeg();
	m_NoDirections	= m_pDoc->getNoDirections();
	m_RoadWidth		= m_NoLanes * m_LaneWidth;
	m_NoTicks		= (int)(m_RoadLength/m_TickStep)+1;

	CSize drawArea;
	drawArea.cy = m_Border_Top + m_Border_Btm + m_RoadWidth;
	drawArea.cx = m_Border_Lhs + m_Border_Rhs + m_RoadLength;
	SetScrollSizes(MM_TEXT, drawArea);

	UpdateDrawRect();
}

void CEvolveTrafficView::UpdateDrawRect()
{
	GetClientRect(&m_ClientRect);
	m_YCoordTop = m_ClientRect.Height()/2 - m_RoadWidth/2;
	
	m_RoadRect.left	  = m_Border_Lhs;								// x-coord of upper left
	m_RoadRect.top	  = m_YCoordTop - m_Border_Top;					// y-coord of upper left
	m_RoadRect.right  = m_RoadLength + m_Border_Rhs;				// x-coord of lower right
	m_RoadRect.bottom = m_YCoordTop + m_RoadWidth + m_Border_Btm;	// y-coord of lower right
}

void CEvolveTrafficView::OnDraw(CDC* dc)
{
	CMemDC pDC(dc);

	DrawRoad(pDC);
	DrawRuler(pDC);
	DrawLegend(pDC);
	
	if(m_bInSimulation)		// we're in a simulation
	{
		M2D VehiclePositions;
		// Only do a step if it's time
		if(m_bTimerHasTicked && m_bAnimate)	// we're not paused and the timer has gone
		{
			for(int i = 0; i < m_SimStepsPerDraw; i++)	// so we can speed it all up
				// note m_bInSimulation will become false if it's the end of the simulation
				VehiclePositions = m_pDoc->m_Sim.doOneTimeStep(&m_bInSimulation);
			m_bTimerHasTicked = false;
		}
		else
			// we're paused or timer hasn't gone yet
			// No need to update sim, just get current positions
			VehiclePositions = m_pDoc->m_Sim.getPositions();

		for (int i = 0; i < VehiclePositions.size(); i++)			// Total number of lanes
		{
			for (int j = 0; j < VehiclePositions[i].size(); j++)	// No vehicles in each lane
					DrawVehicle(pDC, VehiclePositions[i][j]);
		}
		if(!m_bInSimulation)
			MessageBox("The simulation has finished.","EvolveTraffic",MB_OK|MB_ICONINFORMATION);
	}
}

void CEvolveTrafficView::DrawRoad(CMemDC* pDC)
{
	// Prepare brushes
	CPen BlackPen(PS_SOLID, 0, CLR_ROAD_LINES);
	CPen BlackDashPen(PS_DASH, 0, CLR_ROAD_LINES);

	// fill the road surface
	int ypos = m_ClientRect.Height()/2 - m_RoadWidth/2;
	pDC->FillSolidRect(m_Border_Lhs,ypos,m_RoadLength,m_RoadWidth, CLR_ROAD_SURFACE);

	pDC->SelectObject(&BlackPen);
	// Top edge of the road
	ypos = m_ClientRect.Height()/2 - m_RoadWidth/2;
	pDC->MoveTo(m_Border_Lhs,ypos);	
	pDC->LineTo(m_Border_Lhs+m_RoadLength,ypos);
	// bottom edge of the road
	ypos = m_ClientRect.Height()/2 + m_RoadWidth/2;
	pDC->MoveTo(m_Border_Lhs,ypos);	
	pDC->LineTo(m_Border_Lhs+m_RoadLength,ypos);
	// for each lane
	pDC->SelectObject(&BlackDashPen);
	for (int i = 0; i < m_NoLanes-1; i++)
	{	// starting at the bottom, work our way up
		ypos -= m_LaneWidth;
		pDC->MoveTo(m_Border_Lhs,ypos);	
		pDC->LineTo(m_Border_Lhs+m_RoadLength,ypos);
	}

	// Divide carraigeway
	pDC->SelectObject(&BlackPen);
	if(m_NoDirections == 2)
	{
		int nLanesOnBtm = DRIVE_ON_RIGHT ? m_NoLanesDirPos : m_NoLanesDirNeg;
		ypos = m_ClientRect.Height()/2 + m_RoadWidth/2 - m_LaneWidth * nLanesOnBtm;	// need to change
		pDC->MoveTo(m_Border_Lhs,ypos);	
		pDC->LineTo(m_Border_Lhs+m_RoadLength,ypos);
	}

}

void CEvolveTrafficView::DrawRuler(CMemDC* pDC)
{
	int ypos = 	m_ClientRect.Height()/2 + m_RoadWidth/2;
	CPen RulerLinesPen(PS_SOLID, 0, CLR_RULER_LINES);
	pDC->SelectObject(&RulerLinesPen);
	pDC->SetTextColor(CLR_RULER_TEXT);
	pDC->SetBkColor(CLR_BACKGRND);

	for(int i = 0; i < m_NoTicks; i++)
	{
		int xpos = m_Border_Lhs+i*m_TickStep;
		pDC->MoveTo(xpos,ypos);
		pDC->LineTo(xpos,ypos+m_TickLength);
		CString str; str.Format("%d m",i*m_TickStep);
		pDC->TextOut(xpos+5,ypos+10,str);
	}
}

void CEvolveTrafficView::DrawVehicle(CMemDC *pDC, Vehicle* pVeh)
{
	// dir = 0 - Positive x-direction	// dir = 1 - Negative x-direction
	// int dir = pVeh->getDirection();	// KILLED OFF CRAPPY DIR IDENTIFIER
	bool DirPointsRight = pVeh->getDirection();	
	int lane = pVeh->getLane();
	int x = pVeh->getRoadPos() + m_Border_Lhs;	// front of vehicle road position to screen position
	int y = m_ClientRect.Height()/2;			// just to initialize it

	// This works off cumulative lane numbers - from BTM if(DRIVE_ON_RIGHT), else from TOP
	if(DRIVE_ON_RIGHT)
		y = m_YCoordTop + m_RoadWidth - (lane - 0.5) * m_LaneWidth;
	else
		y = m_YCoordTop + (lane - 0.5) * m_LaneWidth;
	
	// and subtract half the vehicle width to get the y pos of the top edge
	y = y - m_VehicleWidth/2;

	// So DirNeg vehicles' position corresponds with the front of the vehicle
	int DirectionFactor = DirPointsRight == true ? -1 : +1;	//int DirectionFactor = dir == 0 ? -1 : +1;
	int cx = DirectionFactor * m_VehicleLengthScale * pVeh->getLength();

	WORD VEH_ID = pVeh->getID();
	CRect VehRect(x,y,x+cx,y+m_VehicleWidth);
	switch(VEH_ID)
	{
		case VEH_ID_CAR:
			pDC->FillSolidRect(VehRect, VEH_COLOR_CAR);
			break;
		case VEH_ID_SMALLTRUCK:
			pDC->FillSolidRect(VehRect, VEH_COLOR_SMALLTRUCK);
			break;
		case VEH_ID_LARGETRUCK:
			pDC->FillSolidRect(VehRect, VEH_COLOR_LARGETRUCK);
			break;
		case VEH_ID_CRANE:
			pDC->FillSolidRect(VehRect, VEH_COLOR_CRANE);
			break;
		case VEH_ID_LOWLOADER:
			pDC->FillSolidRect(VehRect, VEH_COLOR_LOWLOADER);
			break;
		default:
			pDC->FillSolidRect(VehRect, VEH_COLOR_CAR);	// car
	}

/*	// temporary code to identify the front of the vehicle
	x = pVeh->getRoadPos() + m_Border_Lhs;	// road position to screen position;
	CPen BlackPen(PS_SOLID, 0, RGB(0,0,0));
	pDC->SelectObject(&BlackPen);
	pDC->MoveTo(x,y+5);
	pDC->LineTo(x,y-5);
*/
}

void CEvolveTrafficView::DrawLegend(CMemDC *pDC)
{
	pDC->SetTextColor(CLR_LEGEND_TEXT);
	pDC->SetBkColor(CLR_BACKGRND);

	int ypos = m_ClientRect.Height() - m_Border_Btm;
	int xpos = m_Border_Lhs;
	double time = m_pDoc->m_Sim.getCurrentSimTime();

	CString str; str.Format("%.2f s",time);
	CSize strSize = pDC->GetTextExtent(str);	// so text is always inside the window
	pDC->TextOut(xpos,ypos - strSize.cy,str);
}

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficView printing

BOOL CEvolveTrafficView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CEvolveTrafficView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CEvolveTrafficView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficView diagnostics

#ifdef _DEBUG
void CEvolveTrafficView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CEvolveTrafficView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CEvolveTrafficDoc* CEvolveTrafficView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CEvolveTrafficDoc)));
	return (CEvolveTrafficDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CEvolveTrafficView message handlers

void CEvolveTrafficView::OnRunEvolveTraffic() 
{
	if( !m_bInSimulation )
	{
		m_pDoc->initSim();
		m_bTrucksReadIn = true;
		SetTheTimer();
		m_bInSimulation = true;
		m_bAnimate = true;
	}
	else
		MessageBox("Simulation already running", "EvolveTraffic", MB_OK|MB_ICONWARNING);
}

void CEvolveTrafficView::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent==m_TimerID)
	{
		// Set m_bTimerHasTicked so that it gets redrawn
		m_bTimerHasTicked = true;
		// tell windows the view needs redrawn
		// note: the last parameter is the erase flag.
		// if it is TRUE, things will flicker like crazy.
		InvalidateRect(NULL,FALSE);
		//InvalidateRect(&m_RoadRect,FALSE);
	}
	
	CScrollView::OnTimer(nIDEvent);
}

// This function calculates how many simulation steps are to be calculated
// during each refresh of the screen, depending on the user's Time Warp
void CEvolveTrafficView::SetTheTimer()
{
	if( m_bInSimulation )		// we already have a timer
		KillTimer(m_TimerID);	// so kill it

	double SimTimeStep = m_pDoc->getSimTimeStep();
	double NoSimStepsReqd = m_TimeWarp / (DRAW_FRAMES_PER_SEC * SimTimeStep);

	m_SimStepsPerDraw = Round(NoSimStepsReqd);

	// now change the TimeWarp to the rounded value of m_SimStepsPerDraw
	m_TimeWarp = m_SimStepsPerDraw * SimTimeStep * DRAW_FRAMES_PER_SEC;
	
	SetTimer(m_TimerID, m_TimerSpeed, NULL);
}

void CEvolveTrafficView::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);
	
	UpdateDrawRect();	
}

void CEvolveTrafficView::OnToolsPause() 
{
	m_bAnimate = !m_bAnimate;	// so pause/start behaviour

	SetTheTimer();	// in case m_TimeWarp was changed during pause
/*	
	CMenu menu; // Main menu
	CMenu* pContextMenu;
	menu.LoadMenu(IDR_MAINFRAME); // main menu resource ID
	pContextMenu = menu.GetSubMenu(0);
	pContextMenu->EnableMenuItem(ID_RUN_PAUSE, MF_BYCOMMAND|MF_GRAYED);
*/
}

void CEvolveTrafficView::OnConfigSim()
{
	CSimConfigDlg SimDlg;

	SimDlg.m_FileIn			= m_pDoc->getFileIn();
	SimDlg.m_FileOut		= m_pDoc->getFileOut();
	SimDlg.m_FileType		= m_pDoc->getFileType();
	SimDlg.m_RoadLength		= m_pDoc->getRoadLength();
	SimDlg.m_NoLanesDirPos	= m_pDoc->getNoLanesDirPos();
	SimDlg.m_NoLanesDirNeg	= m_pDoc->getNoLanesDirNeg();
	SimDlg.m_SimTimeStep	= (int)(m_pDoc->getSimTimeStep() * 1000);	// seconds to milliseconds
	
	SimDlg.m_AllowLaneChanging			= m_pDoc->getAllowLaneChanging();
	SimDlg.m_TrafFileNoLanesDirPos		= m_pDoc->getTrafFileNoLanesDirPos();
	SimDlg.m_TrafFileNoLanesDirNeg		= m_pDoc->getTrafFileNoLanesDirNeg();
	SimDlg.m_PosOutputDetectorDirPos	= m_pDoc->getPosOutputDetectorDirPos();
	SimDlg.m_PosOutputDetectorDirNeg	= m_pDoc->getPosOutputDetectorDirNeg();

	if(SimDlg.DoModal() == IDOK)
	{
		m_pDoc->setFileIn(			SimDlg.m_FileIn		);
		m_pDoc->setFileOut(			SimDlg.m_FileOut	);
		m_pDoc->setFileType(		SimDlg.m_FileType	);
		m_pDoc->setRoadLength(		SimDlg.m_RoadLength	);
		m_pDoc->setNoLanesDirPos(	SimDlg.m_NoLanesDirPos);
		m_pDoc->setNoLanesDirNeg(	SimDlg.m_NoLanesDirNeg);
		m_pDoc->setSimTimeStep(		(double)SimDlg.m_SimTimeStep / 1000 );	// milliseconds to seconds
		
		m_pDoc->setAllowLaneChanging(		SimDlg.m_AllowLaneChanging );
		m_pDoc->setTrafFileNoLanesDirPos(	SimDlg.m_TrafFileNoLanesDirPos );
		m_pDoc->setTrafFileNoLanesDirNeg(	SimDlg.m_TrafFileNoLanesDirNeg );
		m_pDoc->setPosOutputDetectorDirPos( SimDlg.m_PosOutputDetectorDirPos );
		m_pDoc->setPosOutputDetectorDirNeg( SimDlg.m_PosOutputDetectorDirNeg );

		m_pDoc->UpdateDerivedMembers();
		SettingUpdateRedraw();
	};
}

void CEvolveTrafficView::OnConfigTraf() 
{
	CTrafficConfigDlg TrafDlg;

	TrafDlg.m_IDMParams_Car			= m_pDoc->getIDMParams_Car();
	TrafDlg.m_IDMParams_SmallTruck	= m_pDoc->getIDMParams_SmallTruck();
	TrafDlg.m_IDMParams_LargeTruck	= m_pDoc->getIDMParams_LargeTruck();
	TrafDlg.m_IDMParams_Crane		= m_pDoc->getIDMParams_Crane();
	TrafDlg.m_IDMParams_Lowloader	= m_pDoc->getIDMParams_Lowloader();

	if(TrafDlg.DoModal() == IDOK)
	{
		m_pDoc->setIDMParams_Car		(TrafDlg.m_IDMParams_Car);
		m_pDoc->setIDMParams_SmallTruck	(TrafDlg.m_IDMParams_SmallTruck);
		m_pDoc->setIDMParams_LargeTruck	(TrafDlg.m_IDMParams_LargeTruck);
		m_pDoc->setIDMParams_Crane		(TrafDlg.m_IDMParams_Crane);
		m_pDoc->setIDMParams_Lowloader	(TrafDlg.m_IDMParams_Lowloader);
	}	
}

void CEvolveTrafficView::OnToolsPrefs() 
{
	// we're loosing accuracy in the scaled sizes beecause of rounding

	CPreferencesDlg PrefsDlg;

	PrefsDlg.m_Scale				= m_Scale;							// Don't scale the scale!!
	PrefsDlg.m_TimeWarp				= m_TimeWarp;						// No scale required
	PrefsDlg.m_TickStep				=			m_TickStep;				// No scale required
	
	PrefsDlg.m_LaneWidth			= m_Scale * m_LaneWidth;
	PrefsDlg.m_VehicleWidth			= m_Scale * m_VehicleWidth;
	PrefsDlg.m_VehicleLengthScale	= m_Scale * m_VehicleLengthScale;
	PrefsDlg.m_TickLength			= m_Scale * m_TickLength;
	PrefsDlg.m_Border_Top			= m_Scale * m_Border_Top;
	PrefsDlg.m_Border_Btm			= m_Scale * m_Border_Btm;
	PrefsDlg.m_Border_Lhs			= m_Scale * m_Border_Lhs;
	PrefsDlg.m_Border_Rhs			= m_Scale * m_Border_Rhs;

	if(PrefsDlg.DoModal() == IDOK)
	{
		m_Scale					=	PrefsDlg.m_Scale;
		m_TimeWarp				=	PrefsDlg.m_TimeWarp;			// No scale required
		m_TickStep				=	PrefsDlg.m_TickStep;			// No scale required
		
		m_LaneWidth				=	(PrefsDlg.m_LaneWidth			/m_Scale);
		m_VehicleWidth			=	(PrefsDlg.m_VehicleWidth		/m_Scale);
		m_VehicleLengthScale	=	(PrefsDlg.m_VehicleLengthScale	/m_Scale);
		m_TickLength			=	(PrefsDlg.m_TickLength			/m_Scale);
		m_Border_Top			=	(PrefsDlg.m_Border_Top			/m_Scale);
		m_Border_Btm			=	(PrefsDlg.m_Border_Btm			/m_Scale);
		m_Border_Lhs			=	(PrefsDlg.m_Border_Lhs			/m_Scale);
		m_Border_Rhs			=	(PrefsDlg.m_Border_Rhs			/m_Scale);

		SettingUpdateRedraw();
	}
}

void CEvolveTrafficView::SettingUpdateRedraw()
{
	SetTheTimer();	// Speed up or slow down simulation accordingly
	initRoad();		// re-do the road properties
	Invalidate();	// draw it
}

void CEvolveTrafficView::ScaleChangeUpdate(double factor)
{
	double OldScale = m_Scale;
	m_Scale += factor; // * m_Scale;	// new scale;

	factor = (m_Scale/OldScale);

	// Unscaled variable:	
	//			var_uns = var * OldScale
	// new scaled variable:	
	//			var_new = var_uns / NewScale
	// Both operations together:
	//			var_new = var * (OldScale/NewScale)
	// and allowing for NewScale = OldScale * factor:
	//			var_new = var * (OldScale/ (OldScale * factor) )
	// Since the OldScales cancel, we have:
	//			var_new = var / factor
	// Short and sweet!

	// Parameters that need to be changed only:
	m_LaneWidth			*= factor;
	m_VehicleWidth		*= factor;
	m_VehicleLengthScale*= factor;
	m_TickLength		*= factor;
	m_Border_Top 		*= factor;
	m_Border_Btm 		*= factor;
	m_Border_Lhs 		*= factor;
	m_Border_Rhs 		*= factor;

	// Some checks
	m_VehicleLengthScale < 1 ? 1.0 : m_VehicleLengthScale;	// stops the vehicles disappearing due to rounding!

}

void CEvolveTrafficView::OnToolsZoomin() 
{
	ScaleChangeUpdate(+0.2);		// MAGIC NUMBER - should it be an option?

	SettingUpdateRedraw();
}

void CEvolveTrafficView::OnToolsZoomout() 
{
	ScaleChangeUpdate(-0.2);		// MAGIC NUMBER - should it be an option?

	SettingUpdateRedraw();
}

void CEvolveTrafficView::OnToolsSpeedup() 
{
	m_TimeWarp += 1.0; //* m_TimeWarp;	// MAGIC NUMBER - should it be an option?

	SettingUpdateRedraw();
}

void CEvolveTrafficView::OnToolsSlowdown() 
{
	m_TimeWarp -= 1.0; //0.5 * m_TimeWarp;	// MAGIC NUMBER - should it be an option?

	m_TimeWarp = m_TimeWarp < 1.0 ? 1.0 : m_TimeWarp;	// set real time as minimum 

	SettingUpdateRedraw();
	
}

void CEvolveTrafficView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	
	CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);

	POINT curPoint = GetScrollPosition();
	int deltaX = Round(m_RoadLength/10);	// MAGIC NUMBER - scroll 1/10 of road

	switch(nChar)
	{
		case VK_RIGHT:
			curPoint.x += deltaX;	
			ScrollToPosition(curPoint);
			break;

		case VK_LEFT:
			curPoint.x -= deltaX;
			ScrollToPosition(curPoint);
			break;
	}

}
