
#include "IDM.h"

IDM::IDM()
{
	m_T = 1.2;		// MAGIC NUMBER
	m_A = 1;		// MAGIC NUMBER
	m_B = 2;		// MAGIC NUMBER
	m_S0 = 1;		// MAGIC NUMBER
	m_S1 = 10;		// MAGIC NUMBER
	m_V0 = 80;		// MAGIC NUMBER
	m_Delta = 4;	// MAGIC NUMBER
	m_Polite = 2;	// MAGIC NUMBER
	m_Bias = 2;		// MAGIC NUMBER
	m_DB = 2;		// MAGIC NUMBER

	m_SafeBraking = SAFE_BRAKING;
	m_MinGap = MIN_GAP;
}

IDM::~IDM()
{

}

// IDM driver behaviours

double IDM::update(double vel, double dist)
{
	double accel = dvdt(vel, dist);
	return accel;
}

double IDM::dvdt(double dv, double s)
{
	double s_str = s_star(dv);
	double intel_acc = m_A*(1-pow(m_speed/m_V0,m_Delta));
	double intel_dec = -m_A*pow(s_str/s,2);

	double net_acc = intel_acc + intel_dec;

	return net_acc;
}

double IDM::s_star(double dv)
{
	// Desired gap

	double s_str = m_S0 + m_S1 * sqrt(m_speed/m_V0) 
		+ m_T*m_speed + (m_speed*dv)/(2*sqrt(m_A*m_B));

	return s_str;
}

// For compatibility with the DriverModel base class

void IDM::setVel(double vel)
{
	m_speed = vel;
}

double IDM::getDesiredVel()
{
	return m_V0;
}

////////////////////////////////////////////////////////
////////////////// LANE CHANGING FUNCTION //////////////
////////////////////////////////////////////////////////

double IDM::LaneChange(double GapToFront, double GapToBack, double FrontChangeAccel, 
					   double CurrentBackAccel, double ProposedBackAccel, bool overtake)
{
	double advantage = 0.0;
	double othersDisadv = 0.0;
	double netAdvantage = 0.0;

	// Not really sure about these checking constants
	// but at least they're confined in IDM now!

	// if(GapToFront > m_MinGap)

	if(GapToFront >= m_MinGap && GapToBack >= m_MinGap && ProposedBackAccel >= m_SafeBraking)
	{
		advantage = FrontChangeAccel + (overtake ? -1 : 1) * m_Bias;
		// new line
		othersDisadv = CurrentBackAccel - ProposedBackAccel;
	}

//	if(GapToBack > m_MinGap && ProposedBackAccel > m_SafeBraking)
//		othersDisadv = CurrentBackAccel - ProposedBackAccel;

	if(othersDisadv < 0.0)
		othersDisadv = 0.0;

	netAdvantage = advantage - othersDisadv * m_Polite;

	if(netAdvantage <= m_DB)	// if it's under the threshold
		netAdvantage = 0.0;

	return netAdvantage;
}


////////////////////////////////////////////
// Parameter sets and gets
////////////////////////////////////////////

// Note: unit conversions to be done in here

// Safe time headway, T

void IDM::set_T(double val)
{
	// Unit - input: secs
	//		- output: secs
	m_T = val;
}

double IDM::get_T()
{
	// Unit - input: secs
	//		- output: secs
	return m_T;
}

// Maximum acceleration, A

void IDM::set_A(double val)
{
	// Unit - input: m/s^2
	//		- output: m/s^2
	m_A = val;
}

double IDM::get_A()
{
	// Unit - input: m/s^2
	//		- output: m/s^2	
	return m_A;
}

// Desired decceleration, B

void IDM::set_B(double val)
{
	// Unit - input: m/s^2
	//		- output: m/s^2	
	m_B = val;
}

double IDM::get_B()
{
	// Unit - input: m/s^2
	//		- output: m/s^2	
	return m_B;
}

// Elastic jam distance, S0

void IDM::set_S0(double val)
{
	// Unit - input: m
	//		- output: m	
	m_S0 = val;
}

double IDM::get_S0()
{
	// Unit - input: m
	//		- output: m	
	return m_S0;
}

// Minimum jam distance, S1

void IDM::set_S1(double val)
{
	// Unit - input: m
	//		- output: m	
	m_S1 = val;
}

double IDM::get_S1()
{
	// Unit - input: m
	//		- output: m	
	return m_S1;
}

// Desired velocity, V0

void IDM::set_V0(double val)
{
	// Unit - input: km/h
	//		- output: m/s	
	m_V0 = val / 3.6;	// MAGIC NUMBER = conversion from km/h to m/s
}

double IDM::get_V0()
{
	// Unit - input: m/s
	//		- output: km/h	
	return m_V0 * 3.6;	// MAGIC NUMBER = conversion from m/s to km/h 
}

// Acceleration exponent, Delta

void IDM::set_Delta(double val)
{
	// Unit - input: none
	//		- output: none
	m_Delta = val;
}

double IDM::get_Delta()
{
	// Unit - input: none
	//		- output: none
	return m_Delta;
}

// MOBIL politeness factor

void IDM::set_Polite(double val)
{
	// Unit - input: none
	//		- output: none
	m_Polite = val;
}

double IDM::get_Polite()
{
	// Unit - input: none
	//		- output: none
	return m_Polite;
}

// MOBIL right lane bias

void IDM::set_Bias(double val)
{
	// Unit - input: none
	//		- output: none
	m_Bias = val;
}

double IDM::get_Bias()
{
	// Unit - input: none
	//		- output: none
	return m_Bias;
}

// MOBIL overtaking threshold

void IDM::set_DB(double val)
{
	// Unit - input: none
	//		- output: none
	m_DB = val;
}

double IDM::get_DB()
{
	// Unit - input: none
	//		- output: none
	return m_DB;
}
