#ifndef _ROAD_H
#define _ROAD_H

#include "IDMParameterSet.h"
#include "Direction.h"
#include "TruckGroup.h"

#include "CASTORFile.h"
#include "SAFTFile.h"
#include "Constants.h"
#include <vector>
#include <fstream>

typedef std::vector< std::vector<Vehicle*> > M2D;

	/** A class to represent a road in the simulation */

class Road {

public:
	void setAllowLaneChanging(bool status);
	bool getAllowLaneChanging();
	void setPosOuputDetectorDirPos(int loc);
	void setPosOuputDetectorDirNeg(int loc);
	void setDetectors(std::vector<Detector*> detectors);
	void initTruckGroup(string inFile, string outFile, WORD fileType);
	void setNoLanesDirPos(int nlpos);
	void setNoLanesDirNeg(int nlneg);
	void setNoDirections(int nd);
	void setNoLanes(int nl);
	void setRoadLength(int L);
	void setTrafFileNoLanesDirPos(int nl);
	void setTrafFileNoLanesDirNeg(int nl);

	void setIDMParams_Car		(CIDMParameterSet* Params);
	void setIDMParams_SmallTruck(CIDMParameterSet* Params);
	void setIDMParams_LargeTruck(CIDMParameterSet* Params);
	void setIDMParams_Crane		(CIDMParameterSet* Params);
	void setIDMParams_Lowloader	(CIDMParameterSet* Params);

	double getLength();
	M2D getVehicles();
	//int getTrafFileNoLanesDirPos() const {return m_TrafFileNoLanesDirPos;};
	//int getTrafFileNoLanesDirNeg() const {return m_TrafFileNoLanesDirNeg;};

	void populate(double step, const double curTime);
	double init();

	Road();
	virtual ~Road();

	bool update(double step, const double curTime);

private:
	int m_PosOutputDetectorDirPos;
	int m_PosOutputDetectorDirNeg;
	void MapTrafLaneToSimLane(Vehicle* pVeh);
	void setIDMDriverModel(Vehicle* pVeh);
	CIDMParameterSet* m_pIDMParams_Car;
	CIDMParameterSet* m_pIDMParams_SmallTruck;
	CIDMParameterSet* m_pIDMParams_LargeTruck;
	CIDMParameterSet* m_pIDMParams_Crane;
	CIDMParameterSet* m_pIDMParams_Lowloader;

	Direction DirectionPos;
	Direction DirectionNeg;

	TruckGroup* m_TruckGroup;
	FileHandler* m_FileHandler;
	int m_TrafFileNoLanesDirPos;
	int m_TrafFileNoLanesDirNeg;
	
	std::vector<Truck*> m_BufferVehicles;	// needs to be a vector of vehicles not trucks
	std::vector<Lane*> m_vLanes;
	std::vector<Detector*> m_vDetectorsPos;
	std::vector<Detector*> m_vDetectorsNeg;
	std::string filename;

	bool	m_bEndOfFile;
	bool	m_AllowLaneChanging;
	int		m_NoLanesDirPos;
	int		m_NoLanesDirNeg;
	int		m_NoDirections;
	int		m_NoLanes;
	int		m_RoadLength;

};
#endif
