// SAFTFile.cpp: implementation of the SAFTFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "SAFTFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

	/** Constructor
	 *
	 * @param inputFile The file to read input from
	 * @param outputFile The file to write output to
	 *
	 */
SAFTFile::SAFTFile(std::string inputFile, std::string outputFile)
{
	inFile.open(inputFile.c_str(), std::ios::in);
	outFile.open(outputFile.c_str(), std::ios::out);
	pTruck = singleTruck;
}

	/** Default destructor */
SAFTFile::~SAFTFile()
{

}

	/** Reads a line representing a truck from a SAFT file
	 *
	 * @return A newly created truck
	 * 
	 * This function reads the next line available from a SAFT format file
	 * and stores the information in a string. If the string is not null, a
	 * truck is created with the properties specified by the line from the
	 * file, and the truck is returned
	 */
Truck* SAFTFile::readLine()
{
	std::string str;

	std::getline(inFile, str, '\n');

	if(str != "")
	{
		Truck* tempTruck = new Truck();
		tempTruck->createSAFTTruck(str);
		return tempTruck;
	}

	return NULL;
}

	/** Writes a line representing a truck to a SAFT file
	 *
	 * @param pVeh The vehicle to write to file
	 *
	 * This function takes a vehicle as a parameter, and calls this vehicle's
	 * doSAFTDATA function, in order to obtain a string which represents all
	 * of the vehicle's physical properties. It then writes this string to a 
	 * SAFT format file
	 */
void SAFTFile::writeLine(Vehicle *pVeh, const double curTime)
{
	pVeh->doSAFTData(pTruck,curTime);
	outFile << singleTruck << std::endl;
}
