
#include "Sim.h"

	/** Default Constructor */
Sim::Sim()
{
	m_CurrentSimTime = 0.0;
}

	/** Default Destructor */
Sim::~Sim()
{
	
}

	/** Initialises the simulation 
	 *
	 * This function takes data which has been passed from the Graphical User
	 * Interface, and initialises the Road object with this data.
	 */
void Sim::init()
{
	m_Road.setNoDirections(m_NoDirections);
	m_Road.setNoLanes(m_NoLanes);
	m_Road.setNoLanesDirPos(m_NoLanesDirPos);
	m_Road.setNoLanesDirNeg(m_NoLanesDirNeg);
	m_Road.setRoadLength(m_RoadLength);

	m_Road.initTruckGroup(m_FileIn, m_FileOut, m_FileType);
	m_CurrentSimTime = m_Road.init();
}

	/** Steps the simulation forward by one timestep
	 *
	 * @param pInSimulation Whether or not the simulation is still running
	 * @return All of the vehicles currently on the Road
	 *
	 * This function is called while the simulation is running. It firstly
	 * populates the road each step, by calling Road.populate(), and then
	 * updates each vehicle on the road by calling Road.update().
	 * In addition to this, the function return all of the vehicles currently
	 * on the road.
	 */
M2D Sim::doOneTimeStep(bool* pInSimulation)
{
	m_Road.populate(m_SimTimeStep,m_CurrentSimTime);
	
	bool SimOver = m_Road.update(m_SimTimeStep, m_CurrentSimTime);
	if(SimOver)
		(*pInSimulation) = false;	// we're no longer in a simulation
	 
	m_vVehicles = m_Road.getVehicles();
	m_CurrentSimTime += m_SimTimeStep;

	return m_vVehicles;
}

	/** Gets all the vehicles on the road
	 *
	 * @return All the vehicles on the road
	 */
M2D Sim::getPositions()
{
	return m_vVehicles;
}

	/** Gets the length of the road
	 *
	 * @return The length of the road
	 */
double Sim::getRoadLength()
{
	return m_Road.getLength();
}


// Set functions for the initialization

	/** Sets the file to read input from
	 *
	 * @param file The file to read input from
	 */
void Sim::setFileIn(string file)
{
	m_FileIn = file;
}

	/** Sets the file to write output to
	 *
	 * @param file The file to write output to
	 */
void Sim::setFileOut(string file)
{
	m_FileOut = file;
}

	/** Sets the file type of the input
	 *
	 * @param fileType The file type of the input
	 */
void Sim::setFileType(WORD fileType)
{
	m_FileType = fileType;
}

	/** Sets the number of lanes in the positive direction
	 *
	 * @param nlpos The number of lanes in the positive direction
	 */
void Sim::setNoLanesDirPos(int nlpos)
{
	m_NoLanesDirPos = nlpos;
}

	/** Sets the number of lanes in the negative direction
	 *
	 * @param nlneg The number of lanes in the negative direction
	 */
void Sim::setNoLanesDirNeg(int nlneg)
{
	m_NoLanesDirNeg = nlneg;
}

	/** Sets the size of the timestep
	 *
	 * @param ts The size of the timestep
	 */
void Sim::setSimTimeStep(double ts)
{
	m_SimTimeStep = ts;
}

	/** Sets the number of directions
	 *
	 * @param nd The number of directions
	 */
void Sim::setNoDirections(int nd)
{
	m_NoDirections = nd;
}

	/** Sets the number of lanes
	 *
	 * @param nl The number of lanes
	 */
void Sim::setNoLanes(int nl)
{
	m_NoLanes = nl;
}

	/** Sets the length of the road
	 *
	 * @param L The length of the road
	 */
void Sim::setRoadLength(int L)
{
	m_RoadLength = L;
}

	/** Gets the current time of the simulation
	 *
	 * @return The current time of the simulation
	 */
double Sim::getCurrentSimTime()
{
	return m_CurrentSimTime;
}

	/** Gets the simulation's road object
	 *
	 * @return The simulation's road object
	 */
Road* Sim::getRoad()
{
	return &m_Road;
}
