// TrafficConfigDlg.cpp : implementation file
//

#include "stdafx.h"
#include "EvolveTraffic.h"
#include "TrafficConfigDlg.h"

#include "GridCtrl_src/GridCellCombo.h"
#include "GridCtrl_src/GridCellNumeric.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTrafficConfigDlg dialog


CTrafficConfigDlg::CTrafficConfigDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTrafficConfigDlg::IDD, pParent)
{
    m_sDistributions.Add("Exponential");
    m_sDistributions.Add("Log-Normal");
    m_sDistributions.Add("Gamma");
	m_sDistributions.Add("Gumbel");
	m_sDistributions.Add("Poisson");
	m_sDistributions.Add("GEV");
	m_sDistributions.Add("Normal");
	m_sDistributions.Add("Constant");

	m_sColumnHeaders.Add("Distribution");
	m_sColumnHeaders.Add("Location");
	m_sColumnHeaders.Add("Scale");
	m_sColumnHeaders.Add("Shape");

	m_sRowHeaders.Add("T");
	m_sRowHeaders.Add("a");
	m_sRowHeaders.Add("b");	
	m_sRowHeaders.Add("s0");
	m_sRowHeaders.Add("s1");
	m_sRowHeaders.Add("v0");
	m_sRowHeaders.Add("Delta");
	m_sRowHeaders.Add("Politeness");
	m_sRowHeaders.Add("Right bias");
	m_sRowHeaders.Add("DB");

	m_sCmbOptions.Add("Class 0 - Cars");
	m_sCmbOptions.Add("Class 1 - Small Truck");
	m_sCmbOptions.Add("Class 2 - Large Truck");
	m_sCmbOptions.Add("Class 3 - Crane");
	m_sCmbOptions.Add("Class 4 - Low-loader");
	
	m_nFixCols = 1;	// MAGIC NUMBER for row & column heads
	m_nFixRows = 1;
	m_nCols = m_sColumnHeaders.GetSize() + m_nFixCols;
	m_nRows = m_sRowHeaders.GetSize() + m_nFixRows;

	//{{AFX_DATA_INIT(CTrafficConfigDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CTrafficConfigDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTrafficConfigDlg)
	DDX_Control(pDX, IDC_CMB_CLASSCOPY, m_cmbVehicleClassCopy);
	DDX_Control(pDX, IDC_CMB_CLASSDEFINE, m_cmbVehicleClassDefine);
	DDX_Control(pDX, IDC_GRID, m_Grid);             // associate the grid window with a C++ object
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTrafficConfigDlg, CDialog)
	//{{AFX_MSG_MAP(CTrafficConfigDlg)
	ON_BN_CLICKED(IDC_BTN_COPY, OnBtnCopy)
	ON_CBN_SELCHANGE(IDC_CMB_CLASSCOPY, OnSelchangeCmbClassCopy)
	ON_CBN_SELCHANGE(IDC_CMB_CLASSDEFINE, OnSelchangeCmbClassDefine)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_GRID, OnGridEndEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTrafficConfigDlg message handlers

BOOL CTrafficConfigDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	TRY {
		m_Grid.SetRowCount(m_nRows);
		m_Grid.SetColumnCount(m_nCols);
		m_Grid.SetFixedRowCount(m_nFixRows);
		m_Grid.SetFixedColumnCount(m_nFixCols);
	}
	CATCH (CMemoryException, e)
	{
		e->ReportError();
		return FALSE;
	}
	END_CATCH
	
	SetGridHeadings();
	SetCells();
	m_Grid.ExpandToFit();	
	
	m_pCurrentIDMParamSet = &m_IDMParams_Car;
	LoadParamsIntoGrid();

	// Load ComboBox text dynamically
	for(int i = 0; i < m_sCmbOptions.GetSize(); i++)
	{
		m_cmbVehicleClassDefine.AddString(m_sCmbOptions.GetAt(i));
		m_cmbVehicleClassCopy.AddString(m_sCmbOptions.GetAt(i));
	}
	// Select the first vehicle class
	m_cmbVehicleClassDefine.SelectString(0,m_sCmbOptions.GetAt(0));

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTrafficConfigDlg::SetGridHeadings()
{
	int row = 0;
	int col = 0;

	// Set fixed column text
	for (col = 1; col < m_nCols; col++)
		m_Grid.SetItemText(row,col,m_sColumnHeaders.GetAt(col-1));

	// Set fixed row text
	col = 0;
	for (row = 1; row < m_nRows; row++)
		m_Grid.SetItemText(row,col,m_sRowHeaders.GetAt(row-1));
}

void CTrafficConfigDlg::SetCells()
{
	for (int row = 1; row < m_nRows; row++)
	{
		// MAGIC NUMBER is column = 1
		int col = 1;
		m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellCombo));	
	    CGridCellCombo *pCell = (CGridCellCombo*) m_Grid.GetCell(row,col);
		pCell->SetOptions(m_sDistributions);
		pCell->SetStyle(CBS_DROPDOWN); //CBS_DROPDOWN, CBS_DROPDOWNLIST, CBS_SIMPLE
	}

	for (row = 1; row < m_nRows; row++)
	{
		for (int col = 2; col < m_nCols; col++)
		{
			m_Grid.SetCellType(row,col, RUNTIME_CLASS(CGridCellNumeric));
			m_Grid.GetCell(row,col)->SetFormat(DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_NOPREFIX|DT_END_ELLIPSIS);
			CGridCellNumeric *pCell = (CGridCellNumeric*)m_Grid.GetCell(row,col);
			pCell->SetFlags(CGridCellNumeric::Real | CGridCellNumeric::Negative);
		}
	}
}

void CTrafficConfigDlg::LoadParamsIntoGrid()
{
	// First, clear the grid
//	CCellRange range(m_nFixRows,m_nFixCols,m_Grid.GetRowCount()-m_nFixRows, m_Grid.GetColumnCount()-m_nFixCols);
//	m_Grid.ClearCells( range );

	LoadRow(m_nFixRows);
	LoadRow(m_nFixRows+1);
	LoadRow(m_nFixRows+2);
	LoadRow(m_nFixRows+3);
	LoadRow(m_nFixRows+4);
	LoadRow(m_nFixRows+5);
	LoadRow(m_nFixRows+6);
	LoadRow(m_nFixRows+7);
	LoadRow(m_nFixRows+8);
	LoadRow(m_nFixRows+9);

	m_Grid.Refresh();
}

void CTrafficConfigDlg::LoadRow(int row)
{
	CParameter* pParam = MapRowToParameter(row);
	
	int col = m_nFixCols;
	CString txt = MapDistributionID( pParam->getDistribution()->getDistributionID() );
	m_Grid.SetItemText(row,col,txt);
	
	CGridCellNumeric* pCell1 = (CGridCellNumeric *)(m_Grid.GetCell(row,col+1));
	CGridCellNumeric* pCell2 = (CGridCellNumeric *)(m_Grid.GetCell(row,col+2));
	CGridCellNumeric* pCell3 = (CGridCellNumeric *)(m_Grid.GetCell(row,col+3));
	pCell1->SetNumber(pParam->getDistribution()->getLocation());
	pCell2->SetNumber(pParam->getDistribution()->getScale());
	pCell3->SetNumber(pParam->getDistribution()->getShape());
}

void CTrafficConfigDlg::OnBtnCopy() 
{
	// Once something is chosen
	if( m_pCurrentIDMParamSet != NULL && m_pCopyIDMParamSet != NULL)
	{
		// and they are not the same objects
		if( m_pCurrentIDMParamSet != m_pCopyIDMParamSet )
		{
			// copy them
			*m_pCurrentIDMParamSet = *m_pCopyIDMParamSet;
			LoadParamsIntoGrid();
		}
	}
}

void CTrafficConfigDlg::OnSelchangeCmbClassCopy() 
{
	int iSelect = m_cmbVehicleClassCopy.GetCurSel();
	m_pCopyIDMParamSet = MapIDMParamSet(iSelect);	
}

void CTrafficConfigDlg::OnSelchangeCmbClassDefine() 
{
	// Load chosen paramset into m_pCurrentIDMParamSet
	int iSelect = m_cmbVehicleClassDefine.GetCurSel();
	m_pCurrentIDMParamSet = MapIDMParamSet(iSelect);

	LoadParamsIntoGrid();
}

// This function handles the GVN_ENDLABELEDIT messages
void CTrafficConfigDlg::OnGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult)
{
	// if change is ok, then *pResult 0, else *pResult -1
	
	// pItem is the cell that has just been edited
    NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
    
    int row = pItem->iRow;
	int col = pItem->iColumn;

	if(col == m_nFixCols)	// is the distriubtion column
	{
		CString strCell = m_Grid.GetCell(row,col)->GetText();
		if(strCell != "")
		{
			// data is ok
			SetParamData(row,col,strCell);
			*pResult = 0;
		}	
		else
			*pResult = -1;
	}
	else
	{
		// no need to verify CNumericCells since will always be a number
		// even deleting a cell's data results in a zero
		double val = ((CGridCellNumeric *)(m_Grid.GetCell(row,col)))->GetNumber();
		SetParamData(row,col,val);
		*pResult = 0;
	}
}

void CTrafficConfigDlg::SetParamData(int row, int col, CString str)
{
	CParameter* currentParam = MapRowToParameter(row);
	WORD dist = MapDistributionString(str);
	currentParam->getDistribution()->setDistributionID(dist);
}

void CTrafficConfigDlg::SetParamData(int row, int col, double val)
{
	CParameter* currentParam = MapRowToParameter(row);
	
	// this means we're independent of m_nFixCols in the cases
	col = col - m_nFixCols;		
	switch(col)
	{
		case 1:
			currentParam->getDistribution()->setLocation(val);
			break;
		case 2:
			currentParam->getDistribution()->setScale(val);
			break;
		case 3:
			currentParam->getDistribution()->setShape(val);
			break;
		default:
			currentParam->getDistribution()->setLocation(val);
	}
}

/////////////////////////////////////////////////////////////////
///		MAPPING FUNCTIONS FOR TEXT/OBJECT TO ID & BACK

CParameter* CTrafficConfigDlg::MapRowToParameter(int row)
{
	// This function assumes m_pCurrentIDMParamSet != NULL
	// This function assumes the order of the parameters does not change

	// this means we're independent of m_nFixCols in the cases
	row = row - m_nFixRows;	
	switch(row)
	{
		case 0:		return m_pCurrentIDMParamSet->get_T();
		case 1:		return m_pCurrentIDMParamSet->get_A();
		case 2:		return m_pCurrentIDMParamSet->get_B();
		case 3:		return m_pCurrentIDMParamSet->get_S0();
		case 4:		return m_pCurrentIDMParamSet->get_S1();
		case 5:		return m_pCurrentIDMParamSet->get_V0();
		case 6:		return m_pCurrentIDMParamSet->get_Delta();
		case 7:		return m_pCurrentIDMParamSet->get_Polite();
		case 8:		return m_pCurrentIDMParamSet->get_Bias();
		case 9:		return m_pCurrentIDMParamSet->get_DB();
		default:	return m_pCurrentIDMParamSet->get_T();	// T
	}
}

CIDMParameterSet* CTrafficConfigDlg::MapIDMParamSet(int iSelect)
{
	// Note that this function requires the index of the
	// m_sCmbOptions text to not change. There is no way 
	// to avoid this since some function must map the txt
	// string to the m-IDMParams_ classes

	switch(iSelect)
	{
		case 0:		return &m_IDMParams_Car;
		case 1:		return &m_IDMParams_SmallTruck;
		case 2:		return &m_IDMParams_LargeTruck;
		case 3:		return &m_IDMParams_Crane;
		case 4:		return &m_IDMParams_Lowloader;
		default:	return &m_IDMParams_Car;
	}
}

CString CTrafficConfigDlg::MapDistributionID(WORD distID)
{
	// Note this function reqires the order of the distributions 
	// to not change
	switch(distID)
	{
		case DIST_EXPONENTIAL:	return m_sDistributions.GetAt(0);
		case DIST_LOGNORMAL:	return m_sDistributions.GetAt(1);
		case DIST_GAMMA:		return m_sDistributions.GetAt(2);
		case DIST_GUMBEL:		return m_sDistributions.GetAt(3);
		case DIST_POISSON:		return m_sDistributions.GetAt(4);
		case DIST_GEV:			return m_sDistributions.GetAt(5);
		case DIST_NORMAL:		return m_sDistributions.GetAt(6);
		case DIST_CONST:		return m_sDistributions.GetAt(7);
		default:				return m_sDistributions.GetAt(7);	// constant
	}
}

WORD CTrafficConfigDlg::MapDistributionString(CString dist)
{
	// Note this function reqires the order of the distributions 
	// to not change
	int i = 0;
	while(dist != m_sDistributions.GetAt(i))
		i++;

	switch(i)
	{
		case 0:		return DIST_EXPONENTIAL;
		case 1:		return DIST_LOGNORMAL;
		case 2:		return DIST_GAMMA;		 
		case 3:		return DIST_GUMBEL;			 
		case 4:		return DIST_POISSON;			 
		case 5:		return DIST_GEV;			 
		case 6:		return DIST_NORMAL;			 
		case 7:		return DIST_CONST;			 
		default:	return DIST_CONST;
	}
}