// Truck.cpp: implementation of the Truck class.
//
//////////////////////////////////////////////////////////////////////

#include "Truck.h"
#include <strstrea.h>

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

	/** Default Constructor*/
Truck::Truck(){
/*
	HEAD	= 0;	DAY		= 0;	MONTH	= 0;	YEAR	= 5;	HOUR	= 0;
	MIN		= 0;	SEC		= 0;	hNDT	= 0;	SPEED	= 0;	GVW		= 0;
	LENGTH	= 0;	NAXLES	= 0;	DIRN	= 0;	LANE	= 0;	TRNS	= 0;*/

}

	/** Default Destructor*/
Truck::~Truck()
{
	for(int i = 0; i < axles.size(); i++)
		delete axles.at(i);
}

	/** Parses the Truck's data into an array*/
void Truck::returnTruckData(int truckData[])
{

	truckData[0] = head;
	truckData[1] = day;
	truckData[2] = month;
	truckData[3] = year;
	truckData[4] = hour;
	truckData[5] = min;
	truckData[6] = sec;
	truckData[7] = hndt;	
	truckData[8] = vel;
	truckData[9] = GVW;
	truckData[10] = len;
	truckData[11] = noAxles;
	truckData[12] = dir;	//m_DirPointsToRight == true ? 1 : 2;
	truckData[13] = lane;
	truckData[14] = trns;

	for(int i = 0; i <= 33; i++)
	{
		if( (truckData[i] < 0) || (truckData[i] > 9999) ) truckData[i] = 0;
	}
}

	/** Gets the spacing of a given axle
	 *
	 * The space of a given axle to the next axle
	 * @param i the axle to operate upon
	 * @return the space between this axle and the next
	 */

double Truck::getAS(int i){
	return (axles.at(i)->getSpacing());
}

	/** Gets the weight of a given axle
	 * @param i the axle to operate upon
	 * @return the weight of the axle
	 */

double Truck::getAW(int i){
	return (axles.at(i)->getWeight());
}

	/** Gets the current time of the truck in seconds
	 *
	 * @return The time of the truck in seconds
	 */
double Truck::getTime() const
{
	// working off a 10 month year & a 25 day month
	int s_per_hr = 3600;
	int s_per_day = 3600*24;

	int no_days = (year - 5) * MTS_PER_YR * DAYS_PER_MT + (month - 1) * DAYS_PER_MT + (day-1);
	double time = no_days * s_per_day + hour * s_per_hr + min * 60 + sec + (double)hndt/100;

	return time;
}

bool Truck::operator<(const Truck& x)
{
	double a = this->getTime();
	double b = x.getTime();
	return a < b;
}

bool Truck::operator==(Truck& x)
{
	const int N = 33;
	int truck1[N]; int truck2[N];
	returnTruckData(truck1);
	x.returnTruckData(truck2);

	bool same = true; int i = 0;
	while(same && i < N)
	{
		if(truck1[i] != truck2[i])
			same = false;
		i++;
	}
	return same;
}

	/** Gets the truck's number of axles
	 * @return the truck's number of axles
	 */

int Truck::getNoAxles(){
	return noAxles;
}

	/** Gets the GVW of the truck
	 * @return the GVW of the truck
	 */

double Truck::getGVW(){
	return GVW;
}

	/** Sets the spacing of a given axle
	 *
	 * The space of a given axle to the next axle
	 * @param i the axle to operate upon
	 * @param s the desired spacing
	 */

void Truck::setAS(int i, double s){
	axles.at(i)->setSpacing(s);
}

	/** Sets the weight of a given axle
	 * @param i the axle to operate upon
	 * @param w the desired weight
	 */
void Truck::setAW(int i, double w){
	axles.at(i)->setWeight(w);
}

	/**Sets the GVW of the truck to a given weight
	 * @param weight the desired GVW
	 */
void Truck::setGVW(double weight){
	GVW = weight;
}

	/**Sets the truck's number of axles to a given number
	 * @param axNo the desired number of axles
	 *
	 * This function sets the Truck's number of Axles and current differentiates
	 * between large and small trucks based on the number of Axles
	 */
void Truck::setNoAxles(int axNo){

	ID = (axNo < 4) ? VEH_ID_SMALLTRUCK : VEH_ID_LARGETRUCK;

	noAxles = axNo;

	for(int i = 0; i < noAxles; i++){
		Axle* axle = new Axle();
		axles.push_back(axle);
	}
}


	/** Creates a truck from a given CASTOR string
	 *
	 * @param data The CASTOR string of data
	 *
	 * This function takes in a string from a CASTOR file and creates a truck
	 * based on the properties supplied by the data string
	 */
void Truck::createCASTORTruck(std::string data)
{
	m_x = 0.0;
	roadPos = m_x;

	head	= atoi( data.substr(0,4).c_str() );
	day		= atoi( data.substr(4,2).c_str() );
	month	= atoi( data.substr(6,2).c_str() );
	year	= atoi( data.substr(8,2).c_str() );
	hour	= atoi( data.substr(10,2).c_str() );
	min		= atoi( data.substr(12,2).c_str() );
	sec		= atoi( data.substr(14,2).c_str() );
	hndt	= atoi( data.substr(16,2).c_str() );
	vel		= atoi( data.substr(18,3).c_str() );
	GVW		= atoi( data.substr(21,4).c_str() );
	len		= atoi( data.substr(25,3).c_str() );
	noAxles	= atoi( data.substr(28,1).c_str() );
	dir		= atoi( data.substr(29,1).c_str() );
	lane	= atoi( data.substr(30,1).c_str() );
	trns	= atoi( data.substr(31,3).c_str() );

	m_DirPos = dir == 1 ? true : false;		// Required for compatibility with legacy

	/* Length = length/10 for dm to meters 
	   Vel = vel / 10 for dm/s to meters/second
	   GVW * 0.981 for KG/100 to KN
	   Trns = trns/10 for dm to meters */	

	len = len/10;
	vel = vel/10;
	GVW *= 0.981;
	trns = trns/10;

	setNoAxles(noAxles);
	double W;
	double S;
	int j = 32;

	for(int i = 0; i < noAxles; i++){
		j += 2;
		W = atoi( data.substr(j,3).c_str() );
		j += 3;
		S = atoi( data.substr(j,2).c_str() );
		setAW(i, W);
		setAS(i, S);
	}

	tdelay = 0.0;
	overhangFront = 0.0;
	overhangBack = 0.0;

}

	/** Creates a truck from a given SAFT string
	 *
	 * @param data The SAFT string of data
	 *
	 * This function takes in a string from a SAFT file and creates a truck
	 * based on the properties supplied by the data string
	 */
void Truck::createSAFTTruck(std::string data)
{
	m_x = 0.0;
	roadPos = m_x;

	order	= atoi( data.substr(0,5).c_str() );
	head	= 20005;
	day		= atoi( data.substr(10,2).c_str() );
	month	= atoi( data.substr(12,2).c_str() );
	year	= atoi( data.substr(14,2).c_str() );
	hour	= atoi( data.substr(16,2).c_str() );
	min		= atoi( data.substr(18,2).c_str() );
	sec		= atoi( data.substr(20,2).c_str() );
	hndt	= atoi( data.substr(22,2).c_str() );
	vel		= atoi( data.substr(24,3).c_str() );
	GVW		= atoi( data.substr(27,4).c_str() );
	len		= atoi( data.substr(31,3).c_str() );
	noAxles	= atoi( data.substr(34,1).c_str() );

	lane = 1;
	dir = 1;	//m_DirPointsToRight = true;

	/* Length = length/10 for dm to meters 
	   Vel = vel / 10 for dm/s to meters/second */	

	len = len/10;
	vel = vel/10;

	setNoAxles(noAxles);
	double W;
	double S;
	int j = 33;

	for(int i = 0; i < noAxles; i++){
		j += 2;
		W = atoi( data.substr(j,3).c_str() );
		setAW(i, W);

		if(i == noAxles-1) break;

		j += 3;
		S = atoi( data.substr(j,2).c_str() );
		setAS(i, S);
	}

	tdelay = 0.0;
	overhangFront = 0.0;
	overhangBack = 0.0;
}

	/** Prepares a truck for printing to a CASTOR file
	 *
	 * @param pTruck The string of data to represent the truck
	 *
	 * This function takes in a string, and writes all of its properties to
	 * the string in CASTOR format, in order to prepare for printing of the 
	 * truck
	 */
void Truck::doCASTORData(char* pTruck, const double curTime)
{
	setTime(curTime);

	int velocity = vel*10;
	int grossWeight = GVW/0.981;
	int length = len*10;
	int transPos = trns*10;

	ostrstream oFile( pTruck, sizeof(pTruck)*78 );

	oFile.width(4);	oFile << head;
	oFile.width(2);	oFile << day;
	oFile.width(2);	oFile << month;
	oFile.width(2);	oFile << year;
	oFile.width(2);	oFile << hour;
	oFile.width(2);	oFile << min;
	oFile.width(2);	oFile << sec;
	oFile.width(2);	oFile << hndt;
	oFile.width(3);	oFile << velocity;
	oFile.width(4);	oFile << grossWeight;
	oFile.width(3);	oFile << length;
	oFile.width(1);	oFile << noAxles;
	oFile.width(1);	oFile << dir;
	oFile.width(1);	oFile << lane;
	oFile.width(3);	oFile << transPos;

	int j = 3;
	int i;
	for(i = 0; i < noAxles; i++)
	{
		oFile.width(j);	oFile << this->getAW(i);
		j = 2;

		if(i == 8) break;

		oFile.width(j);	oFile << this->getAS(i);
		j = 3;
	}

	for(i = noAxles; i < 9; i++)
	{
		oFile.width(j);	oFile << "0";
		j = 2;

		if(i == 8) break;

		oFile.width(j);	oFile << "0";
		j = 3;
	}

	oFile << ends;
}

	/** Prepares a truck for printing to a SAFT file
	 *
	 * @param pTruck The string of data to represent the truck
	 *
	 * This function takes in a string, and writes all of its properties to
	 * the string in SAFT format, in order to prepare for printing of the 
	 * truck
	 */
void Truck::doSAFTData(char* pTruck, const double curTime)
{
	setTime(curTime);

	int velocity = vel*10;
	int length = len*10;

	ostrstream oFile( pTruck, sizeof(pTruck)*78 );

	oFile.fill('0');
	oFile.width(5);	oFile << order;
	oFile.width(5);	oFile << head;
	oFile.width(2);	oFile << day;
	oFile.width(2);	oFile << month;
	oFile.width(2);	oFile << year;
	oFile.width(2);	oFile << hour;
	oFile.width(2);	oFile << min;
	oFile.width(2);	oFile << sec;
	oFile.width(2);	oFile << hndt;
	oFile.fill(' ');
	oFile.width(3);	oFile << velocity;
	oFile.width(4);	oFile << GVW;
	oFile.width(3);	oFile << length;
	oFile.width(1);	oFile << noAxles;

	int j = 3;
	int i;
	for(i = 0; i < noAxles; i++)
	{
		oFile.width(j);	oFile << this->getAW(i);
		j = 2;

		if(i == noAxles-1) break;

		oFile.width(j);	oFile << this->getAS(i);
		j = 3;
	}

	oFile << ends;
}

void Truck::setTime(const double t)
{
	double temp = 0.0;
	temp = t;

	const int mpy = MTS_PER_YR;	const int dpm = DAYS_PER_MT;	// for brevity only

	year = (int)(t/(mpy*dpm*24*3600));
	temp = t - year * mpy * dpm * 24 * 3600;	month	= (int)(temp/(dpm * 24 * 3600));
	temp = temp - month * dpm * 24 * 60 * 60;	day		= (int)(temp/(24 * 3600));		
	temp = temp - day * 24 * 60 * 60;			hour	= (int)(temp/(3600));
	temp = temp - hour * 60 * 60;				min		= (int)(temp/(60));
	temp = temp - min * 60;						sec		= (int)(temp);
	temp = temp - sec;							hndt	= (int)(100 * temp);

	// take account of rounding
	head	= 1001;			// MAGIC NUMBERS
	day		= day + 1;
	month	= month + 1;
	year	= year + 8;
}
